DEFAULT_ARG_TEMPLATE = """    <li>
        <b>{name} : <i>{type}, {default_arg}</i></b>
        <ul style='list-style: none'>
            <li>description</li>
        </ul>
    </li>
"""

TEMPLATE = """    <li>
        <b>{name} : <i>{type}</i></b>
        <ul style='list-style: none'>
            <li>description</li>
        </ul>
    </li>
"""
